@@ -38,7 +38,7 @@
 #include <assert.h>
 
 Kig::Kig()
-  : KParts::MainWindow( 0L, "Kig" )
+  : KParts::MainWindow( 0L, "Kig" ), m_part( 0 )
 {
   // set the shell's ui resource file
   setXMLFile("kigui.rc");
@@ -48,7 +48,13 @@ Kig::Kig()
   // this routine will find and load our Part.  it finds the Part by
   // name which is a bad idea usually.. but it's alright in this
   // case since our Part is made for this Shell
-  KLibFactory *factory = KLibLoader::self()->factory("libkigpart");
+
+  // we use globalLibrary() because if we use python scripting, then
+  // we need the python symbols to be exported, in order for python to
+  // be able to load its dll modules..
+  KLibrary* library = KLibLoader::self()->globalLibrary("libkigpart");
+  KLibFactory* factory = 0;
+  if ( library ) factory = library->factory();
   if (factory)
   {
       // now that the Part is loaded, we cast it to a Part to get
@@ -127,7 +133,9 @@ void Kig::readProperties(KConfig* config)
 
 void Kig::load(const KURL& url)
 {
-  if ( m_part->openURL( url ) ) m_recentFilesAction->addURL( url );
+  // we check for m_part not being 0, because in the case of us not
+  // finding our library, we would otherwise get a crash...
+  if ( m_part && m_part->openURL( url ) ) m_recentFilesAction->addURL( url );
 }
 
 void Kig::fileNew()
@@ -162,7 +170,10 @@ void Kig::openURL(const KURL& url)
 
 void Kig::optionsConfigureKeys()
 {
-  KKeyDialog::configureKeys(actionCollection(), "kigui.rc");
+  KKeyDialog dlg( true, this );
+  dlg.insert( actionCollection() );
+  dlg.insert( m_part->actionCollection() );
+  (void) dlg.configure( true );
 }
 
 void Kig::optionsConfigureToolbars()
