@@ -70,13 +70,39 @@ using namespace std;
 
 // export this library...
 typedef KParts::GenericFactory<KigDocument> KigDocumentFactory;
-K_EXPORT_COMPONENT_FACTORY ( libkigpart, KigDocumentFactory );
+K_EXPORT_COMPONENT_FACTORY ( libkigpart, KigDocumentFactory )
 
 KAboutData* KigDocument::createAboutData()
 {
   return kigAboutData( "kigpart", I18N_NOOP( "KigPart" ) );
 }
 
+class SetCoordinateSystemAction
+  : public KAction
+{
+  KigDocument& md;
+  uint mn;
+public:
+  SetCoordinateSystemAction( const QString text, KigDocument& d,
+                             uint i, KActionCollection* parent );
+  void slotActivated();
+};
+
+SetCoordinateSystemAction::SetCoordinateSystemAction(
+  const QString text, KigDocument& d,
+  uint i, KActionCollection* parent )
+  : KAction( text, 0, 0, 0, parent, 0 ),
+    md( d ), mn( i )
+{
+}
+
+void SetCoordinateSystemAction::slotActivated()
+{
+  CoordinateSystem* sys = CoordinateSystemFactory::build( mn );
+  assert( sys );
+  md.history()->addCommand( KigCommand::changeCoordSystemCommand( md, sys ) );
+}
+
 KigDocument::KigDocument( QWidget *parentWidget, const char *,
 			  QObject *parent, const char *name,
 			  const QStringList& )
@@ -126,6 +152,15 @@ void KigDocument::setupActions()
   (void) KStdAction::print( this, SLOT( filePrint() ), actionCollection() );
   (void) KStdAction::printPreview( this, SLOT( filePrintPreview() ), actionCollection() );
 
+  // selection actions
+  aSelectAll = KStdAction::selectAll(
+    this, SLOT( slotSelectAll() ), actionCollection() );
+  aDeselectAll = KStdAction::deselect(
+    this, SLOT( slotDeselectAll() ), actionCollection() );
+  aInvertSelection = new KAction(
+    i18n( "Invert Selection" ), "", 0, this,
+    SLOT( slotInvertSelection() ), actionCollection(),
+    "edit_invert_selection" );
 
   // we need icons...
   KIconLoader* l = KGlobal::iconLoader();
@@ -165,34 +200,57 @@ void KigDocument::setupActions()
   KigExportManager::instance()->addMenuAction( this, m_widget->realWidget(),
                                                actionCollection() );
 
-  KAction* a = KStdAction::zoomIn( m_widget, SLOT( zoomIn() ), actionCollection() );
+  KAction* a = KStdAction::zoomIn( m_widget, SLOT( slotZoomIn() ),
+                                   actionCollection() );
   a->setToolTip( i18n( "Zoom in on the document" ) );
   a->setWhatsThis( i18n( "Zoom in on the document" ) );
 
-  a = KStdAction::zoomOut( m_widget, SLOT( zoomOut() ),
-                                  actionCollection() );
+  a = KStdAction::zoomOut( m_widget, SLOT( slotZoomOut() ),
+                           actionCollection() );
   a->setToolTip( i18n( "Zoom out of the document" ) );
   a->setWhatsThis( i18n( "Zoom out of the document" ) );
 
-  a = KStdAction::fitToPage( m_widget, SLOT( recenterScreen() ),
-                                         actionCollection() );
+  a = KStdAction::fitToPage( m_widget, SLOT( slotRecenterScreen() ),
+                             actionCollection() );
+  // grr.. why isn't there an icon for this..
+  a->setIcon( "new" );
   a->setToolTip( i18n( "Recenter the screen on the document" ) );
   a->setWhatsThis( i18n( "Recenter the screen on the document" ) );
 
+#ifdef KDE_IS_VERSION
+#if KDE_IS_VERSION(3,1,90)
+#define KIG_PART_CPP_STD_FULLSCREEN_ACTION
+#endif
+#endif
+#ifdef KIG_PART_CPP_STD_FULLSCREEN_ACTION
+  a = KStdAction::fullScreen( m_widget, SLOT( toggleFullScreen() ), actionCollection() );
+#else
   tmp = l->loadIcon( "window_fullscreen", KIcon::User );
   a = new KAction(
     i18n( "Full Screen" ), tmp, CTRL+SHIFT+Key_F,
     m_widget, SLOT( toggleFullScreen() ),
-    actionCollection(), "view_fullscreen" );
+    actionCollection(), "fullscreen" );
+#endif
   a->setToolTip( i18n( "View this document full-screen." ) );
   a->setWhatsThis( i18n( "View this document full-screen." ) );
 
+  // TODO: an icon for this..
+  tmp = l->loadIcon( "new", KIcon::User );
   a = new KAction(
-    i18n( "Select Shown Area" ), 0, m_widget, SLOT( zoomRect() ),
+    i18n( "Select Shown Area" ), tmp, 0, m_widget, SLOT( zoomRect() ),
     actionCollection(), "view_select_shown_rect" );
   a->setToolTip( i18n( "Select the area that you want to be shown in the window." ) );
   a->setWhatsThis( i18n( "Select the area that you want to be shown in the window." ) );
-};
+
+  // select coordinate system KActionMenu..
+  KActionMenu* am =
+    new KActionMenu( i18n( "&Set Coordinate System" ), actionCollection(),
+                     "settings_set_coordinate_system" );
+  QStringList csnames = CoordinateSystemFactory::names();
+  for ( uint i = 0; i < csnames.size(); ++i )
+    am->insert( new SetCoordinateSystemAction( csnames[i], *this, i,
+                                               actionCollection() ) );
+}
 
 void KigDocument::setupTypes()
 {
@@ -206,7 +264,7 @@ void KigDocument::setupTypes()
     aActions.push_back( ret );
     ret->plug( this );
   };
-};
+}
 
 KigDocument::~KigDocument()
 {
@@ -236,17 +294,8 @@ KigDocument::~KigDocument()
   MacroList* macrolist = MacroList::instance();
   macrolist->save( macrolist->macros(), typesDir + "macros.kigt" );
 
-  //  this leads to crashes, cause sometimes an object deletes its
-  //  parents, and we try to delete it a second time..
-  // delete_all( mObjs.begin(), mObjs.end() );
-  while ( ! mObjs.empty() )
-  {
-    Object* o = mObjs.back();
-    mObjs.erase( mObjs.end() - 1 );
-    delete o;
-  };
-
-  mObjs.clear();
+  // objects get deleted automatically, when mobjsref gets
+  // destructed..
 
   delete_all( aActions.begin(), aActions.end() );
   aActions.clear();
@@ -254,14 +303,24 @@ KigDocument::~KigDocument()
   // cleanup
   delete mcoordsystem;
   delete mMode;
+  delete mhistory;
 }
 
 bool KigDocument::openFile()
 {
+  QFileInfo fileinfo( m_file );
+  if ( ! fileinfo.exists() )
+  {
+    KMessageBox::sorry( widget(),
+                        i18n( "The file \"%1\" you tried to open does not exist.  "
+                              "Please verify that you entered the correct path." ).arg( m_file ),
+                        i18n( "File Not Found" ) );
+    return false;
+  };
+
   // m_file is always local, so we can use findByPath instead of
   // findByURL...
   KMimeType::Ptr mimeType = KMimeType::findByPath ( m_file );
-  QFile file;
   kdDebug() << k_funcinfo << "mimetype: " << mimeType->name() << endl;
   KigFilter* filter = KigFilters::instance()->find( mimeType->name() );
   if ( !filter )
@@ -286,7 +345,7 @@ bool KigDocument::openFile()
   setModified(false);
   mhistory->clear();
 
-  Objects tmp = calcPath( mObjs );
+  Objects tmp = calcPath( objects() );
   tmp.calc( *this );
   emit recenterScreen();
 
@@ -320,12 +379,12 @@ bool KigDocument::saveFile()
     return true;
   }
   return false;
-};
+}
 
 void KigDocument::addObject(Object* o)
 {
   mhistory->addCommand( KigCommand::addCommand( *this, o ) );
-};
+}
 
 void KigDocument::addObjects( const Objects& os )
 {
@@ -334,9 +393,9 @@ void KigDocument::addObjects( const Objects& os )
 
 void KigDocument::_addObject( Object* o )
 {
-  mObjs.upush( o );
+  mobjsref.addParent( o );
   setModified(true);
-};
+}
 
 void KigDocument::delObject(Object* o)
 {
@@ -344,13 +403,13 @@ void KigDocument::delObject(Object* o)
   Objects all = o->getAllChildren();
   all.upush(o);
   mhistory->addCommand( KigCommand::removeCommand( *this, all ) );
-};
+}
 
 void KigDocument::_delObjects( const Objects& o )
 {
   for ( Objects::const_iterator i = o.begin(); i != o.end(); ++i )
   {
-    mObjs.remove( *i );
+    mobjsref.delParent( *i );
     (*i)->setSelected( false );
   };
   setModified( true );
@@ -358,19 +417,20 @@ void KigDocument::_delObjects( const Objects& o )
 
 void KigDocument::_delObject(Object* o)
 {
-  mObjs.remove( o );
+  mobjsref.delParent( o );
   o->setSelected( false );
   setModified(true);
-};
+}
 
 Objects KigDocument::whatAmIOn(const Coordinate& p, const KigWidget& w ) const
 {
   Objects tmp;
   Objects nonpoints;
-  for ( Objects::const_iterator i = mObjs.begin(); i != mObjs.end(); ++i )
+  const Objects objs = objects();
+  for ( Objects::const_iterator i = objs.begin(); i != objs.end(); ++i )
   {
     if(!(*i)->contains(p, w) || !(*i)->shown() || !(*i)->valid()) continue;
-    if ( (*i)->hasimp( ObjectImp::ID_PointImp ) ) tmp.push_back( *i );
+    if ( (*i)->hasimp( PointImp::stype() ) ) tmp.push_back( *i );
     else nonpoints.push_back( *i );
   };
   std::copy( nonpoints.begin(), nonpoints.end(), std::back_inserter( tmp ) );
@@ -381,23 +441,25 @@ Objects KigDocument::whatIsInHere( const Rect& p, const KigWidget& w )
 {
   Objects tmp;
   Objects nonpoints;
-  for ( Objects::iterator i = mObjs.begin(); i != mObjs.end(); ++i )
+  const Objects objs = objects();
+  for ( Objects::const_iterator i = objs.begin(); i != objs.end(); ++i )
   {
     if(! (*i)->inRect( p, w ) || !(*i)->shown() || ! (*i)->valid() ) continue;
-    if ((*i)->hasimp( ObjectImp::ID_PointImp ) ) tmp.push_back( *i );
+    if ((*i)->hasimp( PointImp::stype() ) ) tmp.push_back( *i );
     else nonpoints.push_back( *i );
   };
   std::copy( nonpoints.begin(), nonpoints.end(), std::back_inserter( tmp ) );
   return tmp;
-};
+}
 
 Rect KigDocument::suggestedRect() const
 {
   bool rectInited = false;
   Rect r(0.,0.,0.,0.);
-  for (Objects::const_iterator i = mObjs.begin(); i != mObjs.end(); ++i )
+  const Objects objs = objects();
+  for (Objects::const_iterator i = objs.begin(); i != objs.end(); ++i )
   {
-    if ( (*i)->shown() && (*i)->hasimp( ObjectImp::ID_PointImp ) )
+    if ( (*i)->shown() && (*i)->hasimp( PointImp::stype() ) )
     {
       if( !rectInited )
       {
@@ -434,7 +496,8 @@ void KigDocument::setMode( KigMode* m )
 
 void KigDocument::_addObjects( const Objects& os )
 {
-  mObjs |= os;
+  os.calc( *this );
+  mobjsref.addParents( os );
   setModified( true );
 }
 
@@ -475,13 +538,13 @@ KCommandHistory* KigDocument::history()
 
 void KigDocument::delObjects( const Objects& os )
 {
-  Objects tos;
-  tos = os;
-  for ( Objects::const_iterator i = os.begin(); i != os.end(); ++i )
-    tos.upush( (*i)->getAllChildren() );
-  Objects dos = os;
+  Objects tos = os.getAllChildren();
+  tos |= os;
+
+  const Objects objs = objects();
+  Objects dos;
   for ( Objects::iterator i = tos.begin(); i != tos.end(); ++i )
-    if ( mObjs.contains( *i ) )
+    if ( objs.contains( *i ) )
       dos.upush( *i );
   if ( dos.empty() ) return;
   mhistory->addCommand( KigCommand::removeCommand( *this, dos ) );
@@ -503,7 +566,7 @@ void KigDocument::unplugActionLists()
   unplugActionList( "user_line_types" );
   unplugActionList( "user_other_types" );
   unplugActionList( "user_types" );
-};
+}
 
 void KigDocument::plugActionLists()
 {
@@ -514,7 +577,7 @@ void KigDocument::plugActionLists()
   plugActionList( "user_line_types", aMNewLine );
   plugActionList( "user_other_types", aMNewOther );
   plugActionList( "user_types", aMNewAll );
-};
+}
 
 void KigDocument::emitStatusBarText( const QString& text )
 {
@@ -585,8 +648,8 @@ void KigDocument::doneMode( KigMode* d )
 
 void KigDocument::setObjects( const Objects& os )
 {
-  assert( mObjs.empty() );
-  mObjs = os;
+  assert( objects().empty() );
+  mobjsref.setParents( os );
 }
 
 void KigDocument::setCoordinateSystem( CoordinateSystem* cs )
@@ -725,7 +788,7 @@ void KigDocument::filePrintPreview()
 void KigDocument::filePrint()
 {
   KPrinter printer;
-  if ( printer.setup( m_widget ) )
+  if ( printer.setup( m_widget, i18n("Print Geometry") ) )
   {
     doPrint( printer );
   };
@@ -758,3 +821,28 @@ void KigDocument::doPrint( KPrinter& printer )
   painter.drawGrid( coordinateSystem() );
   painter.drawObjects( objects() );
 }
+
+const Objects KigDocument::objects() const
+{
+  return mobjsref.parents();
+}
+
+const Objects KigDocument::allObjects() const
+{
+  return getAllParents( objects() );
+}
+
+void KigDocument::slotSelectAll()
+{
+  mMode->selectAll();
+}
+
+void KigDocument::slotDeselectAll()
+{
+  mMode->deselectAll();
+}
+
+void KigDocument::slotInvertSelection()
+{
+  mMode->invertSelection();
+}
