@@ -26,6 +26,8 @@
 #include "../objects/object.h"
 #include "../kig/kig_part.h"
 
+#include "config.h"
+
 #include <qfile.h>
 #include <qtextstream.h>
 #include <qdom.h>
@@ -37,7 +39,7 @@ GUIActionList* GUIActionList::instance()
 {
   static GUIActionList l;
   return &l;
-};
+}
 
 GUIActionList::~GUIActionList()
 {
@@ -226,14 +228,14 @@ Macro::~Macro()
 bool MacroList::save( Macro* m, const QString& f )
 {
   return save( myvector<Macro*>( m ), f );
-};
+}
 
 bool MacroList::save( const myvector<Macro*>& ms, const QString& f )
 {
   QDomDocument doc( "KigMacroFile" );
 
   QDomElement docelem = doc.createElement( "KigMacroFile" );
-  docelem.setAttribute( "Version", "0.5.0" );
+  docelem.setAttribute( "Version", KIGVERSION );
   docelem.setAttribute( "Number", ms.size() );
 
   for ( uint i = 0; i < ms.size(); ++i )
@@ -285,7 +287,7 @@ bool MacroList::load( const QString& f, myvector<Macro*>& ret, const KigDocument
     return loadNew( main, ret, kdoc );
   else
     return loadOld( main, ret, kdoc );
-};
+}
 
 bool MacroList::loadNew( const QDomElement& docelem, myvector<Macro*>& ret, const KigDocument& )
 {
@@ -360,7 +362,6 @@ bool MacroList::loadOld( const QDomElement& docelem, myvector<Macro*>& ret, cons
     // requirements a macro has..
 
     Objects given;
-    Objects givendata;
 
     QDomElement e = hierel.firstChild().toElement();
     for ( ; ! e.isNull(); e = e.nextSibling().toElement() )
@@ -372,30 +373,23 @@ bool MacroList::loadOld( const QDomElement& docelem, myvector<Macro*>& ret, cons
       if(tmp.isNull()) return false;
       QCString type = tmp.latin1();
 
-      Objects data;
-      Object* r = randomObjectForType( type, data );
-      data.calc( doc );
+      Object* r = randomObjectForType( type );
+      r->parents().calc( doc );
       r->calc( doc );
-      copy( data.begin(), data.end(), back_inserter( givendata ) );
       given.push_back( r );
     };
 
     assert( e.attribute( "given" ) != "true" );
-    Objects os( given.begin(), given.end() );
+    ReferenceObject ref;
     Objects final;
 
-    if ( !parseOldObjectHierarchyElements( e, os, final, doc ) ) return false;
+    if ( !parseOldObjectHierarchyElements( e, given, ref,
+                                           final, doc ) ) return false;
 
     assert( !final.empty() );
 
     ObjectHierarchy hierarchy( given, final.front() );
 
-    delete_all( os.begin(), os.end() );
-    os.clear();
-    final.clear();
-    delete_all( givendata.begin(), givendata.end() );
-    givendata.clear();
-
     MacroConstructor* ctor =
       new MacroConstructor( hierarchy, name, QString::null );
     GUIAction* act = new ConstructibleAction( ctor, 0 );
